@@ -30,9 +30,13 @@
 
 void
 img_save_slideshow( img_window_struct *img,
-					const gchar       *output );
+					const gchar       *output,
+					gboolean		   relative );
 
 void
 img_load_slideshow( img_window_struct *img,
 					const gchar       *input );
+
+void
+img_append_slides_from( img_window_struct *img, const gchar *input );
 #endif
