@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009 Giuseppe Torelli <colossus73@gmail.com>
+ *  Copyright (c) 2009-2019 Giuseppe Torelli <colossus73@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,11 @@
  */
 
 #include "support.h"
+#include "audio.h"
+#include "video_formats.h"
 #include <glib/gstdio.h>
 
+extern gchar *img_get_audio_filetype(gchar *);
 static gboolean img_plugin_is_loaded(img_window_struct *, GModule *);
 
 GtkWidget *img_load_icon(gchar *filename, GtkIconSize size)
@@ -33,7 +36,7 @@ GtkWidget *img_load_icon(gchar *filename, GtkIconSize size)
 	g_free (path);
 
 	if (file_pixbuf == NULL)
-		file_image = gtk_image_new_from_stock(GTK_STOCK_MISSING_IMAGE, size);
+		file_image = gtk_image_new_from_icon_name("image-missing", size);
     else
 	{
 		file_image = gtk_image_new_from_pixbuf(file_pixbuf);
@@ -53,11 +56,11 @@ gchar *img_convert_seconds_to_time(gint total_secs)
 }
 
 static void
-sens_cell_func( GtkCellLayout   *layout,
+sens_cell_func( GtkCellLayout   * UNUSED(layout),
 				GtkCellRenderer *cell,
 				GtkTreeModel    *model,
 				GtkTreeIter     *iter,
-				gpointer         data )
+				gpointer         UNUSED(data) )
 {
 	gboolean sensitive = ! gtk_tree_model_iter_has_child( model, iter );
 	g_object_set( cell, "sensitive", sensitive, NULL );
@@ -119,6 +122,12 @@ GtkWidget *_gtk_combo_box_new_text(gboolean pointer)
 	return combo_box;
 }
 
+void img_set_fadeout_duration(img_window_struct *img_struct, gint duration)
+{
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(img_struct->fadeout_duration), duration);
+    img_struct->audio_fadeout = duration;
+}
+
 void img_set_statusbar_message(img_window_struct *img_struct, gint selected)
 {
 	gchar *message = NULL;
@@ -264,7 +273,6 @@ void img_load_available_transitions(img_window_struct *img)
 					if( ! pixbuf )
 					{
 						g_free( pixbuf );
-						g_free( anim );
 
 						tmp = g_build_filename( g_get_home_dir(),
 												".imagination",
@@ -288,7 +296,7 @@ void img_load_available_transitions(img_window_struct *img)
 					g_module_symbol( module, trans[1], &address );
 					gtk_tree_store_append( model, &citer, &piter );
 					gtk_tree_store_set( model, &citer, 0, pixbuf,
-													   1, trans[0],
+													   1, "",
 													   2, address,
 													   3, id,
 													   4, anim,
@@ -311,58 +319,42 @@ static gboolean img_plugin_is_loaded(img_window_struct *img, GModule *module)
 	return (g_slist_find(img->plugin_list,module) != NULL);
 }
 
-void img_show_file_chooser(SexyIconEntry *entry, SexyIconEntryPosition icon_pos,int button,img_window_struct *img)
+void img_show_file_chooser(GtkWidget *entry, GtkEntryIconPosition UNUSED(icon_pos),int UNUSED(button),img_window_struct *img)
 {
 	GtkWidget *file_selector;
 	gchar *dest_dir;
 	gint response;
     GtkFileFilter *video_filter, *all_files_filter;
     gchar *file_extention, *dot_position, *proposed_filename, *file_basename, *stripped_filename;
+    gint i;
 
 	file_selector = gtk_file_chooser_dialog_new (_("Please choose the slideshow project filename"),
 							GTK_WINDOW (img->imagination_window),
 							GTK_FILE_CHOOSER_ACTION_SAVE,
-							GTK_STOCK_CANCEL,
+							"_Cancel",
 							GTK_RESPONSE_CANCEL,
-							GTK_STOCK_SAVE,
+							"_Save",
 							GTK_RESPONSE_ACCEPT,
 							NULL);
 
     /* only video files filter */
     video_filter = gtk_file_filter_new ();
-    switch (img->video_format)
+
+    gtk_file_filter_set_name (video_filter,
+                video_format_list[img->video_format_index].name);
+
+    i = 0;
+    while (video_format_list[img->video_format_index].file_extensions[i] != NULL)
     {
-        default:
-        case 'V':
-        gtk_file_filter_set_name (video_filter, _("VOB (DVD video)") );
-        gtk_file_filter_add_pattern (video_filter, "*.vob");
-        file_extention = ".vob";
-        break;
-
-        case 'M':
-        gtk_file_filter_set_name (video_filter, _("MP4 (MPEG-4)") );
-        gtk_file_filter_add_pattern (video_filter, "*.mp4");
-        file_extention = ".mp4";
-        break;
-
-        case 'O':
-        gtk_file_filter_set_name (video_filter, _("OGV (Theora/Vorbis)") );
-        gtk_file_filter_add_pattern (video_filter, "*.ogv");
-        file_extention = ".ogv";
-        break;
-
-        case 'F':
-        gtk_file_filter_set_name (video_filter, _("FLV (Flash video)") );
-        gtk_file_filter_add_pattern (video_filter, "*.flv");
-        file_extention = ".flv";
-        break;
-
-        case '3':
-        gtk_file_filter_set_name (video_filter, _("3GP (Mobile Phones)") );
-        gtk_file_filter_add_pattern (video_filter, "*.3gp");
-        file_extention = ".3gp";
-        break;
+        file_extention = g_strconcat("*",
+                video_format_list[img->video_format_index].file_extensions[i],
+                                     NULL);
+        gtk_file_filter_add_pattern (video_filter, file_extention);
+        g_free(file_extention);
+        i++;
     }
+    file_extention = video_format_list[img->video_format_index].file_extensions[0];
+
     gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_selector), video_filter);
 
     /* All files filter */
@@ -389,15 +381,14 @@ void img_show_file_chooser(SexyIconEntry *entry, SexyIconEntryPosition icon_pos,
         g_free(stripped_filename);
     }
     else
-    {
         proposed_filename = g_strjoin(NULL, "unknown", file_extention, NULL);
-    }
+ 
     gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER (file_selector), proposed_filename);
     g_free(proposed_filename);
 
     /* set current dir to the project current dir */
     if (img->project_current_dir)
-        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_selector),img->project_current_dir);
+		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_selector),img->project_current_dir);
 
 	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER (file_selector),TRUE);
 	response = gtk_dialog_run (GTK_DIALOG(file_selector));
@@ -419,7 +410,7 @@ img_create_new_slide( void )
 	if( slide )
 	{
 		/* Still part */
-		slide->duration = 1;
+		slide->duration = 1.0;
 
 		/* Transition */
 		slide->path = g_strdup( "0" );
@@ -431,19 +422,30 @@ img_create_new_slide( void )
 
 		/* Subtitles */
 		slide->anim_duration = 1;
-		slide->position = IMG_SUB_POS_MIDDLE_CENTER;
-		slide->placing = IMG_REL_PLACING_EXPORTED_VIDEO;
+		slide->posX = 0;
+		slide->posY = 1;
 		slide->font_desc = pango_font_description_from_string( "Sans 12" );
 		slide->font_color[0] = 0; /* R */
 		slide->font_color[1] = 0; /* G */
 		slide->font_color[2] = 0; /* B */
 		slide->font_color[3] = 1; /* A */
-		/* default: no font color background */
-        slide->font_bgcolor[0] = 1; /* R */
-        slide->font_bgcolor[1] = 1; /* G */
-        slide->font_bgcolor[2] = 1; /* B */
-        slide->font_bgcolor[3] = 1; /* A */
-        
+		/* default: no font color border */
+        slide->font_brdr_color[0] = 1; /* R */
+        slide->font_brdr_color[1] = 1; /* G */
+        slide->font_brdr_color[2] = 1; /* B */
+        slide->font_brdr_color[3] = 1; /* A */
+        /* default: no font background color */
+        slide->font_bg_color[0] = 1; /* R */
+        slide->font_bg_color[1] = 1; /* G */
+        slide->font_bg_color[2] = 1; /* B */
+        slide->font_bg_color[3] = 0; /* A */
+        /* default: no font border color */
+        slide->border_color[0] = 1; /* R */
+        slide->border_color[1] = 1; /* G */
+        slide->border_color[2] = 1; /* B */
+
+		slide->border_width = 1;
+
         /* Load error handling */
         slide->load_ok = TRUE;
         slide->original_filename = NULL;
@@ -463,9 +465,9 @@ img_set_slide_file_info( slide_struct *slide,
 	format = gdk_pixbuf_get_file_info( filename, &width, &height );
 
 	slide->o_filename = g_strdup( filename );
-	slide->r_filename = g_strdup( filename );
+	slide->p_filename = g_strdup( filename );
 	slide->angle = 0;
-	
+
 	slide->resolution = g_strdup_printf( "%d x %d", width, height );
 	slide->type = gdk_pixbuf_format_get_name( format );
 }
@@ -493,9 +495,45 @@ img_set_slide_gradient_info( slide_struct *slide,
 	}
 }
 
+GdkPixbuf *img_set_fade_gradient(img_window_struct *img, gint gradient, slide_struct *slide_info)
+{
+	GdkPixbuf		*pix = NULL;
+	GtkTreeIter 	iter;
+	GtkTreeModel 	*model;
+	gpointer     	address;
+
+	if (gradient == 3)
+	{
+		model = gtk_combo_box_get_model( GTK_COMBO_BOX( img->transition_type ) );
+		/* 10:0 is the path string pointing to the Cross Fade transition */
+		gtk_tree_model_get_iter_from_string( model, &iter, "10:0" );
+		gtk_tree_model_get( model, &iter, 0, &pix,
+										  2, &address,
+										 -1 );
+		slide_info->transition_id = 19;
+		slide_info->render = address;
+		if (slide_info->path)
+		{
+			g_free(slide_info->path);
+			slide_info->path = g_strdup("10:0");
+		}
+	}
+	else
+	{
+		if (slide_info->path)
+		{
+			g_free(slide_info->path);
+			slide_info->path = g_strdup("0");
+		}
+		slide_info->transition_id = -1;
+		slide_info->render = NULL;
+	}
+	return pix;
+}
+
 void
 img_set_slide_still_info( slide_struct      *slide,
-						  gint               duration,
+						  gdouble           duration,
 						  img_window_struct *img )
 {
 	if( slide->duration != duration )
@@ -549,7 +587,7 @@ img_set_slide_ken_burns_info( slide_struct *slide,
 							  gdouble      *points )
 {
 	ImgStopPoint *point;
-	gint          i,
+	gsize          i,
 				  full;
 
 	if( slide->no_points )
@@ -580,102 +618,22 @@ img_set_slide_ken_burns_info( slide_struct *slide,
 		slide->duration = full;
 }
 
-void
-img_set_slide_text_info( slide_struct      *slide,
-						 GtkListStore      *store,
-						 GtkTreeIter       *iter,
-						 const gchar       *subtitle,
-						 gint	            anim_id,
-						 gint               anim_duration,
-						 gint               position,
-						 gint               placing,
-						 const gchar       *font_desc,
-						 gdouble           *font_color,
-                         gdouble           *font_bgcolor,
-						 img_window_struct *img )
-{
-	/* Set the slide text info parameters */
-	if( store && iter )
-	{
-		gboolean flag;
-
-		if( slide->subtitle )
-			g_free( slide->subtitle );
-		slide->subtitle = g_strdup( subtitle );
-
-		flag = ( subtitle ? TRUE : FALSE );
-		gtk_list_store_set( store, iter, 3, flag, -1 );
-	}
-
-	if( ( anim_id > -1 ) && ( anim_id != slide->anim_id ) )
-	{
-		GtkTreeModel *model;
-		gchar        *path;
-		GtkTreeIter   iter;
-
-		path = g_strdup_printf( "%d", anim_id );
-		model = gtk_combo_box_get_model( GTK_COMBO_BOX( img->sub_anim ) );
-		gtk_tree_model_get_iter_from_string( model, &iter, path );
-		g_free( path );
-
-		slide->anim_id = anim_id;
-		gtk_tree_model_get( model, &iter, 1, &slide->anim, -1 );
-
-		/* Sync timings */
-		img_sync_timings( slide, img );
-	}
-
-	if( ( anim_duration > 0 ) && ( anim_duration != slide->anim_duration ) )
-	{
-		slide->anim_duration = anim_duration;
-
-		/* Synchronize timings */
-		img_sync_timings( slide, img );
-	}
-
-	if( ( position > -1 ) && ( position != slide->position ) )
-		slide->position = position;
-
-	if( ( placing > -1 ) && ( placing != slide->placing ) )
-		slide->placing = placing;
-
-	if( font_desc )
-	{
-		if( slide->font_desc )
-			pango_font_description_free( slide->font_desc );
-		slide->font_desc = pango_font_description_from_string( font_desc );
-	}
-
-	if( font_color )
-	{
-		slide->font_color[0] = font_color[0];
-		slide->font_color[1] = font_color[1];
-		slide->font_color[2] = font_color[2];
-		slide->font_color[3] = font_color[3];
-	}
-
-    if( font_bgcolor )
-    {
-        slide->font_bgcolor[0] = font_bgcolor[0];
-        slide->font_bgcolor[1] = font_bgcolor[1];
-        slide->font_bgcolor[2] = font_bgcolor[2];
-        slide->font_bgcolor[3] = font_bgcolor[3];
-    }
-}								
-
 void
 img_free_slide_struct( slide_struct *entry )
 {
 	GList *tmp;
 
-	if( entry->angle != ANGLE_0 )
-		g_unlink( entry->r_filename );
+	img_slide_set_p_filename(entry, NULL);
+
 	g_free(entry->o_filename);
-	g_free(entry->r_filename);
-    g_free(entry->original_filename);
 	g_free(entry->resolution);
 	g_free(entry->type);
 	
+	if (entry->subtitle)
+		g_free(entry->subtitle);
+	if (entry->pattern_filename)
+		g_free(entry->pattern_filename);
+	
 	/* Free stop point list */
 	for( tmp = entry->points; tmp; tmp = g_list_next( tmp ) )
 		g_slice_free( ImgStopPoint, tmp->data );
@@ -708,11 +666,11 @@ img_set_total_slideshow_duration( img_window_struct *img )
 		while( gtk_tree_model_iter_next( model, &iter ) );
 
 		/* Add time of last pseudo slide */
-		if( img->final_transition.render )
+		if( img->final_transition.render && img->bye_bye_transition)
 			img->total_secs += img->final_transition.speed;
 	}
-		
-	time = img_convert_seconds_to_time(img->total_secs);
+	img->total_secs = ceil(img->total_secs);
+	time = img_convert_seconds_to_time((gint)img->total_secs);
 	gtk_label_set_text(GTK_LABEL (img->total_time_data),time);
 	g_free(time);
 
@@ -773,6 +731,7 @@ img_scale_image( const gchar      *filename,
 	gdouble    i_ratio;            /* Export and image aspect ratios */
 	gdouble    skew;               /* Transformation between ratio and
 											 i_ratio */
+	GError     *error = NULL;
 	gboolean   transform = FALSE;  /* Flag that controls scalling */
 
 	/* MAximal distortion values */
@@ -784,8 +743,13 @@ img_scale_image( const gchar      *filename,
 	gdouble min_skew = ( 1 - max_stretch ) * max_crop;
 
 	/* Obtain information about image being loaded */
-	if( ! gdk_pixbuf_get_file_info( filename, &i_width, &i_height ) )
-		return( FALSE );
+	if (!filename) {
+	    return FALSE;
+	}
+	if (!gdk_pixbuf_get_file_info( filename, &i_width, &i_height )) {
+	    g_message("Cannot load image info from %s", filename);
+	    return FALSE;
+	}
 
 	/* How distorted images would be if we scaled them */
 	i_ratio = (gdouble)i_width / i_height;
@@ -890,17 +854,18 @@ img_scale_image( const gchar      *filename,
 			lw = width;
 			lh = (gdouble)height * ( skew + 1 ) / 2;
 		}
-		loader = gdk_pixbuf_new_from_file_at_scale( filename, lw, lh,
-													FALSE, NULL );
+		loader = gdk_pixbuf_new_from_file_at_scale( filename, lw, lh, FALSE, &error );
 	}
 	else
 	{
 		/* Simply load image into pixbuf at size */
-		loader = gdk_pixbuf_new_from_file_at_size( filename, width,
-												   height, NULL );
+		loader = gdk_pixbuf_new_from_file_at_size( filename, width, height, &error );
 	}
-	if( ! loader )
+	if( ! loader ) {
+		g_message( "While loading image data from %s: %s.", filename, error->message );
+		g_error_free( error );
 		return( FALSE );
+	}
 
 	i_width  = gdk_pixbuf_get_width( loader );
 	i_height = gdk_pixbuf_get_height( loader );
@@ -971,18 +936,6 @@ img_scale_image( const gchar      *filename,
 	return( TRUE );
 }
 
-void
-img_set_project_mod_state( img_window_struct *img,
-						   gboolean           modified )
-{
-	if( ( img->project_is_modified ? modified : ! modified ) )
-		return;
-
-	img->project_is_modified = modified;
-
-	/* FIXME: Do any updates here (add "*" to window title, ...). */
-}
-
 void
 img_sync_timings( slide_struct      *slide,
 				  img_window_struct *img )
@@ -1008,10 +961,6 @@ img_sync_timings( slide_struct      *slide,
 		gtk_spin_button_set_value( GTK_SPIN_BUTTON( img->ken_duration ),
 								   point->time );
 	}
-
-	/* Update display */
-	gtk_spin_button_set_value( GTK_SPIN_BUTTON( img->duration ),
-							   slide->anim_duration );
 }
 
 void img_select_nth_slide(img_window_struct *img, gint slide_to_select)
@@ -1086,7 +1035,7 @@ img_scale_gradient( gint              gradient,
 
 	switch( gradient )
 	{
-		case 0: /* Solid color */
+		case 0: case 3: /* Solid and gradient fade */
 			cairo_set_source_rgb( cr, c_start[0], c_start[1], c_start[2] );
 			cairo_paint( cr );
 			break;
@@ -1164,6 +1113,8 @@ img_message (img_window_struct *img,
         gtk_text_buffer_insert(img->message_buffer, &message_end,
                                parsed_message, -1);
 
+        printf ("%s",parsed_message);
+
         free(parsed_message);
 
         if (alert_user)
@@ -1176,3 +1127,315 @@ img_message (img_window_struct *img,
         }
 
 }
+
+void
+img_check_for_encoder(img_window_struct *img)
+{
+	gchar *name = NULL;
+
+	name = g_find_program_in_path("ffmpeg");
+	if (name == NULL)
+	{
+		img_message(img, FALSE, "Using avconv for encoding.\n");
+		img->encoder_name = g_strdup("avconv");
+	}
+	else
+	{
+		img_message(img, FALSE, "Using ffmpeg for encoding.\n");
+		img->encoder_name = name;
+	} 
+}
+
+void img_delete_subtitle_pattern(GtkButton *button, img_window_struct *img)
+{
+	slide_struct 	*slide = img->current_slide;
+	GdkPixbuf 		*pixbuf;
+	GtkWidget		*tmp_image,*fc;
+	GtkIconTheme	*icon_theme;
+
+	if (slide->pattern_filename)
+	{
+		g_free(slide->pattern_filename);
+		slide->pattern_filename = NULL;
+	}
+	icon_theme = gtk_icon_theme_get_default();
+	pixbuf = gtk_icon_theme_load_icon(icon_theme,"image", 20, 0, NULL);
+	tmp_image = gtk_image_new_from_pixbuf(pixbuf);
+	gtk_widget_show(tmp_image);
+	g_object_unref(pixbuf);
+	
+	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(img->pattern_image), tmp_image);
+	gtk_widget_set_sensitive(img->sub_color, TRUE);
+	gtk_widget_set_tooltip_text(img->sub_color, _("Click to choose the font color"));
+
+	fc = gtk_widget_get_toplevel(GTK_WIDGET(button));
+	gtk_widget_destroy(fc);
+	gtk_widget_queue_draw( img->image_area );
+}
+
+void img_preview_with_music(img_window_struct *img, gint offset)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gchar	*cmd_line, *path, *filename, *file, *time;
+	gchar 	**argv;
+	gint argc, selected_slide_number, count = 0;
+	gboolean ret;
+
+	model = gtk_tree_view_get_model( GTK_TREE_VIEW( img->music_file_treeview ) );
+	if( ! gtk_tree_model_get_iter_first( model, &iter ) )
+		return;
+
+	gtk_tree_model_get(GTK_TREE_MODEL(img->music_file_liststore), &iter, 0, &path, 1, &filename, -1);
+
+	file = g_build_filename(path, filename, NULL);
+	g_free(path);
+	g_free(filename);
+
+	path = g_shell_quote( file );
+
+	if (offset > 0)
+	{
+		/* User started the preview NOT from
+		 * first slide, let's calculate the
+		 * total time from the first slide
+		 * up the one before the selected one */
+		 
+		GList *selected;
+		GtkTreeIter iter;
+		GtkTreeIter next_iter;
+		GtkTreeModel *model;
+		slide_struct *info_slide;
+
+		offset = 0;
+		model = GTK_TREE_MODEL( img->thumbnail_model );
+		selected = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(img->active_icon));
+
+		if (selected == NULL)
+			return;
+
+		selected_slide_number = gtk_tree_path_get_indices(selected->data)[0];
+
+		gtk_tree_model_get_iter_first(model, &iter);
+		gtk_tree_model_get(model, &iter,1,&info_slide,-1);
+		
+		offset = (gint) (info_slide->speed + info_slide->duration); 
+		gtk_tree_model_get_iter_first(model, &next_iter);
+		
+		while( count < selected_slide_number - 1)
+		{
+			gtk_tree_model_iter_next( model, &next_iter);
+			gtk_tree_model_get(model, &next_iter, 1 , &info_slide, -1);
+			offset += (gint) (info_slide->speed + info_slide->duration);
+			count++;
+		}
+
+		time = img_convert_seconds_to_time(offset);
+		cmd_line = g_strdup_printf("play -t %s %s trim %s", img_get_audio_filetype(file), path, time);
+		g_free(time);
+	}
+	else
+		cmd_line = g_strdup_printf("play -t %s %s", img_get_audio_filetype(file), path);
+	g_free( path );
+	g_free( file );
+
+	g_shell_parse_argv (cmd_line, &argc, &argv, NULL);
+
+	ret = g_spawn_async_with_pipes( NULL, argv, NULL,
+									G_SPAWN_SEARCH_PATH | 
+									G_SPAWN_DO_NOT_REAP_CHILD |
+									G_SPAWN_STDOUT_TO_DEV_NULL |
+									G_SPAWN_STDERR_TO_DEV_NULL,
+									NULL, NULL, &img->play_child_pid, NULL, NULL, NULL, NULL );
+
+	/* Free argument vector */
+	g_strfreev( argv );
+	
+	/* Let's connect a callback when the first audio file playing ends
+	 * and if there is another audio to be reproduced */
+	if ( ret && gtk_tree_model_iter_next( model, &iter) )
+    {
+		img->next_audio_iter = iter;
+		img->audio_source_id = g_child_watch_add(img->play_child_pid, (GChildWatchFunc) img_play_next_audio_during_preview, img);
+	}
+
+}
+
+void img_play_next_audio_during_preview (GPid UNUSED(pid), gint UNUSED(status), img_window_struct *img)
+{
+	GtkTreeModel *model;
+	gchar	*cmd_line, *path, *filename, *file;
+	gchar 	**argv;
+	gint argc;
+	gboolean ret;
+
+	g_spawn_close_pid( img->play_child_pid );
+	kill (img->play_child_pid, SIGINT);
+
+	gtk_tree_model_get(GTK_TREE_MODEL(img->music_file_liststore), &img->next_audio_iter, 0, &path, 1, &filename, -1);
+
+	file = g_build_filename(path, filename, NULL);
+	g_free(path);
+	g_free(filename);
+
+	path = g_shell_quote( file );
+
+	cmd_line = g_strdup_printf("play -t %s %s", img_get_audio_filetype(file), path);
+	g_free( path );
+	g_free( file );
+
+	g_shell_parse_argv (cmd_line, &argc, &argv, NULL);
+
+	ret = g_spawn_async_with_pipes( NULL, argv, NULL,
+									G_SPAWN_SEARCH_PATH | 
+									G_SPAWN_DO_NOT_REAP_CHILD |
+									G_SPAWN_STDOUT_TO_DEV_NULL |
+									G_SPAWN_STDERR_TO_DEV_NULL,
+									NULL, NULL, &img->play_child_pid, NULL, NULL, NULL, NULL );
+	
+	model = gtk_tree_view_get_model( GTK_TREE_VIEW( img->music_file_treeview ) );
+	
+	if (ret && img->music_preview &&
+			gtk_tree_model_iter_next( model, &img->next_audio_iter ) )
+		img->audio_source_id = g_child_watch_add(img->play_child_pid, (GChildWatchFunc) img_play_next_audio_during_preview, img);
+}
+
+void img_save_relative_filenames(GtkCheckButton *togglebutton, img_window_struct *img)
+{
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton)))
+		img->relative_filenames = TRUE;
+	else
+		img->relative_filenames = FALSE;
+}
+
+void str_replace(gchar *str, const gchar *search, const gchar *replace)
+{
+	for (gchar *cursor = str; (cursor = strstr(cursor, search)) != NULL;)
+	{
+		memmove(cursor + strlen(replace), cursor + strlen(search), strlen(cursor) - strlen(search) + 1);
+        for (gint i = 0; replace[i] != '\0'; i++)
+            cursor[i] = replace[i];
+        cursor += strlen(replace);
+	}
+}
+
+void img_set_text_buffer_tags(img_window_struct *img)
+{
+	GtkTextTag *tag;
+
+	img->tag_table = gtk_text_buffer_get_tag_table(img->slide_text_buffer);
+	tag = gtk_text_tag_new("bold");
+	g_object_set(tag, "weight", PANGO_WEIGHT_BOLD, NULL);
+	gtk_text_tag_table_add (img->tag_table, tag);
+	
+	tag = gtk_text_tag_new("italic");
+	g_object_set(tag, "style", PANGO_STYLE_ITALIC, NULL);
+	gtk_text_tag_table_add (img->tag_table, tag);
+	
+	tag = gtk_text_tag_new("underline");
+	g_object_set(tag, "underline", PANGO_UNDERLINE_SINGLE, NULL);
+	gtk_text_tag_table_add (img->tag_table, tag);
+	
+	tag = gtk_text_tag_new("foreground");
+	gtk_text_tag_table_add (img->tag_table, tag);
+	
+	tag = gtk_text_tag_new("background");
+	gtk_text_tag_table_add (img->tag_table, tag);
+}
+
+void img_store_rtf_buffer_content(img_window_struct *img)
+{
+	GdkAtom		format;
+	GtkTextIter start, end;
+
+	format = gtk_text_buffer_register_serialize_tagset(img->slide_text_buffer, NULL);
+	gtk_text_buffer_get_bounds(img->slide_text_buffer, &start, &end);
+	img->current_slide->subtitle = gtk_text_buffer_serialize(img->slide_text_buffer,
+																img->slide_text_buffer,
+																format,
+																&start, 
+																&end,
+																&img->current_slide->subtitle_length
+																); 
+	gtk_text_buffer_unregister_serialize_format (img->slide_text_buffer, format); 
+}
+
+void img_check_for_rtf_colors(img_window_struct *img, gchar *subtitle)
+{
+	GtkTextTag	*tag;
+	gchar		*dummy, *dummy2, *rgb;
+	gint		len;
+
+	/* foreground */
+	dummy = strstr(subtitle, "foreground-gdk");
+	if (dummy)
+	{
+		dummy = strstr(dummy, "value=");
+		dummy2 = strstr(dummy+7, "\"");
+		len = strlen(dummy+7) - strlen(dummy2);
+
+		rgb = g_new0(gchar, len + 2);
+		rgb[0] = '#';
+		strncpy(rgb+1, dummy+7, len);
+
+		str_replace(rgb, ":", "");
+
+		tag = gtk_text_tag_table_lookup(img->tag_table, "foreground");
+		g_object_set(tag, "foreground-gdk", rgb, NULL);
+		g_free(rgb);
+	}
+	/* background */
+	dummy = strstr(subtitle, "background-gdk");
+	if (dummy)
+	{
+		dummy = strstr(dummy, "value=");
+		dummy2 = strstr(dummy+7, "\"");
+		len = strlen(dummy+7) - strlen(dummy2);
+
+		rgb = g_new0(gchar, len + 2);
+		rgb[0] = '#';
+		strncpy(rgb+1, dummy+7, len);
+		str_replace(rgb, ":", "");
+		
+		tag = gtk_text_tag_table_lookup(img->tag_table, "background");
+		g_object_set(tag, "background-gdk", rgb, NULL);
+		g_free(rgb);
+	}
+}
+
+// Resets info_slide->p_filename to filename and removes the associated temp file if applicable
+void img_slide_set_p_filename(slide_struct *info_slide, gchar *filename)
+{
+    if (!info_slide->p_filename) return;
+    if ( g_strcmp0(info_slide->o_filename, info_slide->p_filename) ) {
+	g_assert(info_slide->flipped || info_slide->angle);
+	g_unlink( info_slide->p_filename );
+    }
+    g_free(info_slide->p_filename);
+    info_slide->p_filename = filename;
+}
+
+void img_update_zoom_variables(img_window_struct *img)
+{
+	/* Store old zoom for calculations */
+	gdouble old_zoom = img->current_point.zoom;
+	gdouble fracx, fracy;
+	gint    tmpoffx, tmpoffy;
+		
+	img->current_point.zoom = gtk_range_get_value( GTK_RANGE(img->ken_zoom) );
+		
+	gdouble aw  = img->video_size[0];
+	gdouble ah  = img->video_size[1];
+	gdouble aw2 = aw / 2;
+	gdouble ah2 = ah / 2;
+
+	fracx = (gdouble)( aw2 - img->current_point.offx ) / ( aw2 * old_zoom );
+	fracy = (gdouble)( ah2 - img->current_point.offy ) / ( ah2 * old_zoom );
+	img->maxoffx = aw * ( 1 - img->current_point.zoom );
+	img->maxoffy = ah * ( 1 - img->current_point.zoom );
+	tmpoffx = aw2 * ( 1 - fracx * img->current_point.zoom );
+	tmpoffy = ah2 * ( 1 - fracy * img->current_point.zoom );
+
+	img->current_point.offx = CLAMP( tmpoffx, img->maxoffx, 0 );
+	img->current_point.offy = CLAMP( tmpoffy, img->maxoffy, 0 );
+}
